﻿using Hotline.Share.Enums.Order;
using XF.Domain.Repository;
using XF.Domain.Repository.Events;

namespace Hotline.Orders.DatabaseEventHandler;

/// <summary>
/// OrderVisit 更新数据后执行事务处理
/// 更新 order 表中的 OrgProcessingResults 和 SeatEvaluate 字段
/// </summary>
public class OrderVisitEventHandler : IUpdateDatabaseEvent<OrderVisit>
{
    private readonly IRepository<OrderVisitDetail> _orderVisitDetailRepository;
    private readonly IOrderRepository _orderRepository;

    public OrderVisitEventHandler(IOrderRepository orderRepository, IRepository<OrderVisitDetail> orderVisitDetailRepository)
    {
        _orderRepository = orderRepository;
        _orderVisitDetailRepository = orderVisitDetailRepository;
    }

    public void OnInsert(OrderVisit entity)
    {
        OrderUpdate(entity);
    }

    public void OnUpdate(OrderVisit entity)
    {
        OrderUpdate(entity);
    }

    private void OrderUpdate(OrderVisit visit)
    {
        if (visit.VisitState != EVisitState.Visited) return;
        var orgDetail = _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == visit.Id && m.VisitTarget == EVisitTarget.Org).First();
        var seatDetail = _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == visit.Id && m.VisitTarget == EVisitTarget.Seat).First();

        _orderRepository.Updateable()
            .SetColumns(m => m.OrgProcessingResults == orgDetail.OrgProcessingResults)
            .SetColumns(m => m.SeatEvaluate == seatDetail.SeatEvaluate)
            .Where(m => m.Id == visit.OrderId)
            .ExecuteCommand();
    }
}
